@@ -6,17 +6,17 @@
  * storing Perl SVs in here.
  */
 
-// when we first allocate a chunk of values, this is the size we use
-#define PLIST_INITIAL_SIZE 2 // 16
+/* when we first allocate a chunk of values, this is the size we use */
+#define PLIST_INITIAL_SIZE 2 /* 16 */
 
 typedef struct PNode {
-  const void* ptr;        // the pointer we are storing; we claim NO OWNERSHIP on it
+  const void* ptr;        /* the pointer we are storing; we claim NO OWNERSHIP on it */
 } PNode;
 
 typedef struct PList {
-  PNode* data;            // a chunk of values
-  unsigned short alen;    // allocated size of chunk
-  unsigned short ulen;    // actual used size in chunk
+  PNode* data;            /* a chunk of values */
+  unsigned short alen;    /* allocated size of chunk */
+  unsigned short ulen;    /* actual used size in chunk */
 } PList;
 
 /*
